#include "URLLockNative.h"
#include <nsXPCOM.h>

/* Implementation file */
NS_IMPL_ISUPPORTS1(UserInfoSupportsVoid, nsISupportsVoid)

/**
 * Constructs a base object that will have its Windows-allocated
 * PUSER_INFO_2 memory block freed with NetApiBufferFree
 */
UserInfoSupportsVoid::UserInfoSupportsVoid(PUSER_INFO_2 userInfo)
{
	this->parent = NULL;
	this->userInfo = userInfo;
}

/**
 * Constructs a child object that will add a usage on the given parent
 * when constructed and remove it when destructed
 */
UserInfoSupportsVoid::UserInfoSupportsVoid(nsISupports *parent, PUSER_INFO_2 userInfo)
{
	this->parent = parent;
	this->userInfo = userInfo;

	// Add a reference on the parent
	parent->AddRef();
}

/**
 * Destructs this object
 */
UserInfoSupportsVoid::~UserInfoSupportsVoid()
{
	// If we have a parent, remove the reference on it
	if (parent) {
		parent->Release();
	}

	// Otherwise, free the memory block using NetApiBufferFree
	else {
		NetApiBufferFree(userInfo);
	}
}

/* readonly attribute unsigned short type; */
NS_IMETHODIMP UserInfoSupportsVoid::GetType(PRUint16 *aType)
{
	// This is a void* type
	*aType = TYPE_VOID;
    return NS_OK;
}

/* [noscript] attribute voidPtr data; */
NS_IMETHODIMP UserInfoSupportsVoid::GetData(void * *aData)
{
	// Return the PUSER_INFO_2 pointer
	*aData = (void *) userInfo;
    return NS_OK;
}
NS_IMETHODIMP UserInfoSupportsVoid::SetData(void * aData)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* string toString (); */
NS_IMETHODIMP UserInfoSupportsVoid::ToString(char **_retval)
{
	// Return the user's name as the toString() value
	*_retval = URLLockNative::convertUnicodeToMozillaASCII(userInfo->usri2_name);
    return NS_OK;
}
